#cython: language_level=3

from .obitypes           cimport const_char_p, \
                                 OBIType_t, \
                                 obiversion_t, \
                                 obiint_t, \
                                 obibool_t, \
                                 obichar_t, \
                                 obifloat_t, \
                                 index_t, \
                                 time_t
from ..capi.obidms       cimport OBIDMS_p
from ..capi.obidmscolumn cimport OBIDMS_column_p


cdef extern from "obiview.h" nogil:

    extern const_char_p VIEW_TYPE_NUC_SEQS
    extern const_char_p NUC_SEQUENCE_COLUMN
    extern const_char_p ID_COLUMN
    extern const_char_p DEFINITION_COLUMN

    struct Column_reference_t :
        const_char_p  column_name
        obiversion_t  version
    
    ctypedef Column_reference_t* Column_reference_p


    struct Obiview_t :
        OBIDMS_p            dms
        const_char_p        name
        OBIDMS_column_p     line_selection
        OBIDMS_column_p     new_line_selection
        OBIDMS_column_p     columns
        bint                read_only
        Column_reference_t  line_selection_reference
        index_t             line_count
        int                 column_count
        const_char_p        comments

    ctypedef Obiview_t* Obiview_p


    struct Obiview_infos_t :
        int                 view_number
        int                 column_count
        index_t             line_count
        const_char_p        name
        const_char_p        created_from
        time_t              creation_date
        bint                all_lines
        Column_reference_t  line_selection
        Column_reference_p  column_references
        const_char_p        view_type
        const_char_p        comments

    ctypedef Obiview_infos_t* Obiview_infos_p


    struct Obiviews_header_t :
        size_t header_size
        size_t views_size
        int    view_count
    
    ctypedef Obiviews_header_t* Obiviews_header_p


    struct Obiviews_infos_all_t :
        Obiviews_header_p    header
        Obiview_infos_p      view_infos

    ctypedef Obiviews_infos_all_t* Obiviews_infos_all_p


    Obiview_p obi_new_view_nuc_seqs(OBIDMS_p dms, const_char_p view_name, Obiview_p view_to_clone, index_t* line_selection, const_char_p comments)

    Obiview_p obi_new_view(OBIDMS_p dms, const_char_p view_name, Obiview_p view_to_clone, index_t* line_selection, const_char_p comments)
    
    Obiview_p obi_new_view_cloned_from_name(OBIDMS_p dms, const_char_p view_name, const_char_p view_to_clone_name, index_t* line_selection, const_char_p comments)

    Obiview_p obi_new_view_nuc_seqs_cloned_from_name(OBIDMS_p dms, const_char_p view_name, const_char_p view_to_clone_name, index_t* line_selection, const_char_p comments)
              
    Obiview_p obi_open_view(OBIDMS_p dms, const_char_p view_name)
    
    Obiviews_infos_all_p obi_read_views(OBIDMS_p dms)

    int obi_unmap_read_views(Obiviews_infos_all_p views)

    int obi_view_add_column(Obiview_p view,
                            const_char_p column_name,
                            obiversion_t version_number,
                            OBIType_t data_type,
                            index_t nb_lines,
                            index_t nb_elements_per_line,
                            const_char_p elements_names,
                            const_char_p indexer_name,
                            const_char_p comments,
                            bint create)

    int obi_view_delete_column(Obiview_p view, const_char_p column_name)
    
    int obi_select_line(Obiview_p view, index_t line_nb)

    int obi_select_lines(Obiview_p view, index_t* line_nbs)

    OBIDMS_column_p obi_view_clone_column(Obiview_p view, const_char_p column_name)
        
    OBIDMS_column_p obi_view_get_column(Obiview_p view, const_char_p column_name)

    OBIDMS_column_p* obi_view_get_pointer_on_column_in_view(Obiview_p view, const_char_p column_name)

    int obi_save_view(Obiview_p view)
    
    int obi_close_view(Obiview_p view)
    
    int obi_save_and_close_view(Obiview_p view)
    
    int obi_column_set_obiint_with_elt_name_in_view(Obiview_p view,
                                                    OBIDMS_column_p column, 
                                                    index_t line_nb, 
                                                    const_char_p element_name, 
                                                    obiint_t value)

    int obi_column_set_obiint_with_elt_idx_in_view(Obiview_p view,
                                                   OBIDMS_column_p column, 
                                                   index_t line_nb, 
                                                   index_t element_idx, 
                                                   obiint_t value)
    
    obiint_t obi_column_get_obiint_with_elt_name_in_view(Obiview_p view,
                                                         OBIDMS_column_p column, 
                                                         index_t line_nb, 
                                                         const_char_p element_name)
    
    obiint_t obi_column_get_obiint_with_elt_idx_in_view(Obiview_p view,
                                                        OBIDMS_column_p column, 
                                                        index_t line_nb, 
                                                        index_t element_idx)

    int obi_column_set_obibool_with_elt_name_in_view(Obiview_p view,
                                                     OBIDMS_column_p column, 
                                                     index_t line_nb, 
                                                     const_char_p element_name, 
                                                     obibool_t value)

    int obi_column_set_obibool_with_elt_idx_in_view(Obiview_p view,
                                                    OBIDMS_column_p column, 
                                                    index_t line_nb, 
                                                    index_t element_idx, 
                                                    obibool_t value)
    
    obibool_t obi_column_get_obibool_with_elt_name_in_view(Obiview_p view,
                                                           OBIDMS_column_p column, 
                                                           index_t line_nb, 
                                                           const_char_p element_name)
    
    obibool_t obi_column_get_obibool_with_elt_idx_in_view(Obiview_p view,
                                                          OBIDMS_column_p column, 
                                                          index_t line_nb, 
                                                          index_t element_idx)

    int obi_column_set_obichar_with_elt_name_in_view(Obiview_p view,
                                                     OBIDMS_column_p column, 
                                                     index_t line_nb, 
                                                     const_char_p element_name, 
                                                     obichar_t value)

    int obi_column_set_obichar_with_elt_idx_in_view(Obiview_p view,
                                                    OBIDMS_column_p column, 
                                                    index_t line_nb, 
                                                    index_t element_idx, 
                                                    obichar_t value)
    
    obichar_t obi_column_get_obichar_with_elt_name_in_view(Obiview_p view,
                                                           OBIDMS_column_p column, 
                                                           index_t line_nb, 
                                                           const_char_p element_name)
    
    obichar_t obi_column_get_obichar_with_elt_idx_in_view(Obiview_p view,
                                                          OBIDMS_column_p column, 
                                                          index_t line_nb, 
                                                          index_t element_idx)

    int obi_column_set_obifloat_with_elt_name_in_view(Obiview_p view,
                                                      OBIDMS_column_p column, 
                                                      index_t line_nb, 
                                                      const_char_p element_name, 
                                                      obifloat_t value)

    int obi_column_set_obifloat_with_elt_idx_in_view(Obiview_p view,
                                                     OBIDMS_column_p column, 
                                                     index_t line_nb, 
                                                     index_t element_idx, 
                                                     obifloat_t value)
    
    obifloat_t obi_column_get_obifloat_with_elt_name_in_view(Obiview_p view,
                                                             OBIDMS_column_p column, 
                                                             index_t line_nb, 
                                                             const_char_p element_name)
    
    obifloat_t obi_column_get_obifloat_with_elt_idx_in_view(Obiview_p view,
                                                            OBIDMS_column_p column, 
                                                            index_t line_nb, 
                                                            index_t element_idx)

    int obi_column_set_obistr_with_elt_name_in_view(Obiview_p view,
                                                    OBIDMS_column_p column, 
                                                    index_t line_nb, 
                                                    const_char_p element_name, 
                                                    const_char_p value)

    int obi_column_set_obistr_with_elt_idx_in_view(Obiview_p view,
                                                   OBIDMS_column_p column, 
                                                   index_t line_nb, 
                                                   index_t element_idx, 
                                                   const_char_p value)
    
    const_char_p obi_column_get_obistr_with_elt_name_in_view(Obiview_p view,
                                                           OBIDMS_column_p column, 
                                                           index_t line_nb, 
                                                           const_char_p element_name)
    
    const_char_p obi_column_get_obistr_with_elt_idx_in_view(Obiview_p view,
                                                            OBIDMS_column_p column, 
                                                            index_t line_nb, 
                                                            index_t element_idx)

    int obi_column_set_obiseq_with_elt_name_in_view(Obiview_p view,
                                                    OBIDMS_column_p column, 
                                                    index_t line_nb, 
                                                    const_char_p element_name, 
                                                    const_char_p value)

    int obi_column_set_obiseq_with_elt_idx_in_view(Obiview_p view,
                                                   OBIDMS_column_p column, 
                                                   index_t line_nb, 
                                                   index_t element_idx, 
                                                   const_char_p value)
    
    char* obi_column_get_obiseq_with_elt_name_in_view(Obiview_p view,
                                                      OBIDMS_column_p column, 
                                                      index_t line_nb, 
                                                      const_char_p element_name)
    
    char* obi_column_get_obiseq_with_elt_idx_in_view(Obiview_p view,
                                                     OBIDMS_column_p column, 
                                                     index_t line_nb, 
                                                     index_t element_idx)

